﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using DevExpress.Data.Filtering;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Editors;
using DevExpress.ExpressApp.Model;
using DevExpress.ExpressApp.TreeListEditors;
using DevExpress.ExpressApp.TreeListEditors.Win;
using DevExpress.ExpressApp.Utils;
using DevExpress.ExpressApp.Win.Controls;
using DevExpress.ExpressApp.Win.Core;
using DevExpress.ExpressApp.Xpo;
using DevExpress.Xpo;
using DevExpress.XtraEditors;
using DevExpress.XtraTreeList;
using DevExpress.XtraTreeList.Nodes;
using Xafari.Base;
using Xafari.Editors.Model;
using Xafari.Model;
using Xafari.Editors.Localization;
using Xafari.Editors;

namespace HierarchyNode.Module.Win.Editors
{
    public class WinHierarchyNodeListEditor : TreeListEditor, IComplexListEditor, ICurrentHierarchyProvider, INodeListEditor
    {

        private XafApplication _application;
        public WinHierarchyNodeListEditor(IModelListView model)
            : base(model)
        {
        }
        private object _rootValue;
        public new object RootValue
        {
            get
            {
                if (base.Adapter == null)
                {
                    return this._rootValue;
                }
                return (base.Adapter.RootValue as Xafari.Base.IHierarchyNode);
            }
            set
            {
                if (this._collectionSource.ObjectSpace.GetObject(value) != this._collectionSource.ObjectSpace.GetObject(this._rootValue))
                {
                    this._rootValue = value;
                    if (base.Adapter != null)
                    {
                        Adapter.RootValue = this._rootValue;
                        if ((base.TreeList != null) && base.TreeList.IsHandleCreated)
                        {
                            base.TreeList.DataSource = null;
                        }
                        this.UpdateCriteria(this.CurrentHierarchyName);
                        base.AssignDataSourceToControl(base.DataSource);
                    }
                }
            }
        }

        private CollectionSourceBase _collectionSource;
        public new void Setup(CollectionSourceBase collectionSource, XafApplication application)
        {
            base.Setup(collectionSource, application);
            this._collectionSource = collectionSource;
            this._application = application;
        }

        protected override TreeNodeInterfaceAdapter CreateAdapter()
        {
            //CriteriaOperator childrenCriteria = null;
            //var iModelCriteriaApplyChildren = this.Model.ModelListViewXafariEditors();
            //if (iModelCriteriaApplyChildren != null && iModelCriteriaApplyChildren.CriteriaApplyChildren)
            //     childrenCriteria = HierarchyNodeInterfaceHelper.GetChildrenCriteriaOperator(this._collectionSource.Criteria);
            //return new HierarchyNodeInterfaceAdapter(this, childrenCriteria);


            var iModelCriteriaApplyChildren = this.Model.ModelListViewXafariEditors();
            if (iModelCriteriaApplyChildren != null && iModelCriteriaApplyChildren.CriteriaApplyChildren)
                return new HierarchyNodeInterfaceAdapter(this, this._collectionSource);
            return new HierarchyNodeInterfaceAdapter(this, null);
        }

        protected override object CreateControlsCore()
        {
            var treelist = //CreateControlsCore1();
            base.CreateControlsCore();
            var baseType = typeof(TreeListEditor);//GetType().BaseType;
            if (baseType != null)
            {
                var methodInfo = baseType.GetMethod("adapter_Changed", BindingFlags.NonPublic | BindingFlags.Instance);
                base.Adapter.Changed -= Delegate.CreateDelegate(typeof(EventHandler), this, methodInfo) as EventHandler;
            }
            base.Adapter.Changed += Adapter_Changed;
            var objectTreeList = treelist as ObjectTreeList;
            if (objectTreeList != null && typeof(Xafari.Base.IHierarchyNode).IsAssignableFrom(this.ObjectType) && this.Model.AllowDragAndDrop())
            {
                objectTreeList.OptionsBehavior.DragNodes = true;
                objectTreeList.DragNodesMode = TreeListDragNodesMode.Standard;
                objectTreeList.DragDrop += objectTreeList_DragDrop;
            }
            return treelist;
        }

        //protected object CreateControlsCore1()
        //{
        //    ObjectTreeList treeList;
        //    TreeNodeInterfaceAdapter adapter;
        //    if (treeList == null)
        //    {
        //        adapter = CreateAdapter();
        //        adapter.ObjectSpace = objectSpace;
        //        adapter.RootValue = rootValue;
        //        adapter.HoldRootValue = holdRootValue;
        //        adapter.Changed += new EventHandler(adapter_Changed);
        //        treeList = CreateTreeListControl();
        //        treeList.Tag = EasyTestTagHelper.FormatTestTable(Name);
        //        treeList.Dock = DockStyle.Fill;
        //        treeList.OptionsView.ShowIndicator = false;
        //        treeList.OptionsView.ShowHorzLines = false;
        //        treeList.OptionsView.ShowVertLines = false;
        //        treeList.OptionsView.ShowButtons = true;
        //        treeList.OptionsView.FocusRectStyle = DrawFocusRectStyle.CellFocus;
        //        treeList.OptionsView.ShowColumns = true;
        //        treeList.OptionsView.ShowRoot = true;
        //        treeList.OptionsBehavior.AutoPopulateColumns = false;
        //        treeList.OptionsBehavior.Editable = false;
        //        treeList.ShowButtonMode = ShowButtonModeEnum.ShowOnlyInEditor;
        //        treeList.OptionsBehavior.KeepSelectedOnClick = true;
        //        treeList.OptionsNavigation.AutoMoveRowFocus = true;
        //        treeList.OptionsSelection.EnableAppearanceFocusedCell = false;
        //        treeList.OptionsSelection.MultiSelect = true;
        //        treeList.HandleCreated += new EventHandler(treeList_HandleCreated);
        //        treeList.FocusedNodeChanged += new FocusedNodeChangedEventHandler(treeList_FocusedNodeChanged);
        //        treeList.BeforeFocusNode += new BeforeFocusNodeEventHandler(treeList_BeforeFocusNode);
        //        treeList.SelectionChanged += new EventHandler(treeList_SelectionChanged);
        //        treeList.DoubleClick += new EventHandler(treeList_DoubleClick);
        //        treeList.KeyDown += new KeyEventHandler(treeList_KeyDown);
        //        treeList.MouseDown += new MouseEventHandler(treeList_MouseDown);
        //        treeList.MouseClick += new MouseEventHandler(treeList_MouseClick);
        //        treeList.MouseMove += new MouseEventHandler(treeList_MouseMove);
        //        treeList.ShowCustomizationForm += new EventHandler(treeList_ShowCustomizationForm);
        //        treeList.HideCustomizationForm += new EventHandler(treeList_HideCustomizationForm);
        //        treeList.VisibleChanged += new EventHandler(treeList_VisibleChanged);
        //        this.ApplyModel();
        //        this.OnPrintableChanged();
        //    }
        //    return treeList;
        //}


        void objectTreeList_DragDrop(object sender, DragEventArgs e)
        {
            if (_application == null) return;
            if (Messaging.GetMessaging(_application).Show(EditorsLocalizer.Active.GetLocalizedString(ItemsStringId.DragDropHierarchyNodeListEditor), EditorsLocalizer.Active.GetLocalizedString(ItemsStringId.MoveHierarchyNodeListEditor), MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                var formatsData = ((DataObject)(e.Data)).GetFormats();
                var objectTreeListNodeDrag = ((DataObject)(e.Data)).GetData(formatsData[0]) as ObjectTreeListNode;
                var objectTreeListNodeDrop =
                    this.TreeList.CalcHitInfo(this.TreeList.PointToClient(new Point(e.X, e.Y))).Node as
                    ObjectTreeListNode;
                var objectTreeList = sender as ObjectTreeList;
                if (objectTreeListNodeDrag == null || objectTreeListNodeDrop == null || objectTreeList == null) return;
                if (objectTreeList.DragNodesMode == TreeListDragNodesMode.Standard && e.Effect != DragDropEffects.Copy)
                {
                    ObjectTreeListNode objectParent = null;
                    if (e.KeyState != 4)
                        objectParent = objectTreeListNodeDrop;
                    else if (objectTreeListNodeDrop.ParentNode != null)
                        objectParent = objectTreeListNodeDrop.ParentNode as ObjectTreeListNode;

                    var objectDrag = objectTreeListNodeDrag.Object as Xafari.Base.IHierarchyNode;
                    var objectDrop = objectParent == null ? null : objectParent.Object as Xafari.Base.IHierarchyNode;
                    if (objectDrag != null) objectDrag.SetHierarchyParent(objectDrop, _currentHierarchyName);

                    var objectSpace = (XPObjectSpace)XPObjectSpace.FindObjectSpaceByObject(objectDrag);
                    objectSpace.CommitChanges();
                    this._collectionSource.ObjectSpace.Refresh();
                }
            }
        }


        private void Adapter_Changed(object sender, EventArgs e)
        {
            this._rootValue = base.Adapter.RootValue;
            base.HoldRootValue = ((HierarchyNodeInterfaceAdapter)base.Adapter).HoldRootValue;
            if ((base.TreeList != null) && base.TreeList.IsHandleCreated)
            {
                base.TreeList.DataSource = null;
            }
            base.AssignDataSourceToControl(base.DataSource);
        }

        public override void BreakLinksToControls()
        {
            if (this.Adapter != null)
                base.Adapter.Changed -= new EventHandler(Adapter_Changed);
            if (this.TreeList != null && typeof(Xafari.Base.IHierarchyNode).IsAssignableFrom(this.ObjectType))
                this.TreeList.DragDrop += objectTreeList_DragDrop;

            base.BreakLinksToControls();
        }

        protected void UpdateCriteria(string currentHierarchyName = null)
        {
            this._collectionSource.BeginUpdateCriteria();
            var parentMember = this._collectionSource.ObjectTypeInfo.Type.GetParentMember(currentHierarchyName);
            if (parentMember != null)
            {
                this._collectionSource.Criteria[HierarchyNodeInterfaceHelper.HierarchyNodePropertyEditorKey] =
                    this.RootValue == null
                        ? CriteriaOperator.Parse(string.Format("[{0}] is null", parentMember.Name))
                        : CriteriaOperator.Parse(
                            string.Format("[{0}]=?", parentMember.Name),
                            this.RootValue);
            }
            this._collectionSource.EndUpdateCriteria();
        }
        public override void Refresh()
        {
            base.Refresh();
            this._collectionSource.ResetCollection();
        }
        private string _currentHierarchyName;
        public string CurrentHierarchyName
        {
            get { return _currentHierarchyName; }
            set
            {
                _currentHierarchyName = value;
                if (!(this._collectionSource is PropertyCollectionSource))
                {
                    var parentProperty = this._collectionSource.ObjectTypeInfo.Type.GetParentMember(value);
                    if (parentProperty != null)
                        this._collectionSource.Criteria[HierarchyNodeInterfaceHelper.HierarchyNodePropertyEditorKey] =
                            CriteriaOperator.Parse(string.Format("{0} is null", parentProperty.Name));
                }
                if (base.TreeList != null)
                    base.TreeList.DataSource = null;
                base.AssignDataSourceToControl(base.DataSource);
            }
        }
    }
}
