﻿using DevExpress.ExpressApp;
using DevExpress.ExpressApp.DC;
using DevExpress.ExpressApp.Model;
using DevExpress.Persistent.Base;
using DevExpress.Persistent.Base.General;
using DevExpress.Persistent.BaseImpl;
using DevExpress.Persistent.Validation;
using DevExpress.Xpo;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using Xafari;
using Xafari.Base;
using Xafari.BC.DC.DynamicProperties;
using Xafari.BC.DynamicProperties;
using Xafari.DC;

namespace IHierarchyNode.Module.BusinessObjects
{

    [DefaultClassOptions]
    public class TreeListObject : BaseObject, ITreeNode
    {
        public TreeListObject(Session session) : base(session) { }

        [Browsable(false)]
        [Association("Parent-Child")]
        public XPCollection<TreeListObject> Children
        {
            get { return GetCollection<TreeListObject>("Children"); }
        }

        [Persistent, Association("Parent-Child")]
        public TreeListObject Parent { get; set; }

        public string Name { get; set; }

        IBindingList ITreeNode.Children
        {
            get { return Children as IBindingList; }
        }

        ITreeNode ITreeNode.Parent
        {
            get { return Parent as ITreeNode; }
        }
    }

    [DefaultClassOptions]
    public class HierarchyNodeObject : BaseObject, Xafari.Base.IHierarchyNode
    {
        public HierarchyNodeObject(Session session) : base(session) { }

        //HierarchyNodeNamePropertyAttribute used in HierarchyNodePersistentHelper
        [HierarchyNodeNameProperty()]
        public String Name { get; set; }

        //PrentPropertyAttribute describe Parent property. You need to set it.
        [Association("Parent - Children")]
        [ParentProperty()]
        public HierarchyNodeObject Parent { get; set; }


        [Association("Parent - Children")] 
        //ChilderPropertyAttribute is obligitory
        [ChildrenProperty()]
        public XPCollection<HierarchyNodeObject> Children { get { return GetCollection<HierarchyNodeObject>("Children"); } }

        #region Implementation of IHierarchyNode

        public string GetHierarchyNodeName(string hierarchyName = null)
        {
            return HierarchyNodePersistentHelper.GetNodeName(this, hierarchyName);
        }

        public string GetHierarchyNodeFullName(string hierarchyName = null)
        {
            return HierarchyNodePersistentHelper.GetNodeFullName(this, hierarchyName);
        }
 
        public void SetHierarchyNodeFullName(string value, string hierarchyName = null)
        {
            HierarchyNodePersistentHelper.SetNodeFullName(this, value, hierarchyName);
        }
 
        public string GetHierarchyNodeFullPath(string hierarchyName = null)
        {
            return HierarchyNodePersistentHelper.GetNodeFullPath(this, hierarchyName);
        }
 
        public void SetHierarchyNodeFullPath(string value, string hierarchyName = null)
        {
            HierarchyNodePersistentHelper.SetNodeFullPath(this, value, hierarchyName);
        }

        public Xafari.Base.IHierarchyNode GetHierarchyParent(string hierarchyName = null)
        {
            return HierarchyNodePersistentHelper.GetParent(this, hierarchyName);
        }

        public void SetHierarchyParent(Xafari.Base.IHierarchyNode parent, string hierarchyName = null)
        {
            HierarchyNodePersistentHelper.SetParent(this, parent, hierarchyName);
        }
 
        public IList GetHierarchyChildren(string hierarchyName = null)
        {
            return HierarchyNodePersistentHelper.GetChildren(this, hierarchyName);
        }
 
        #endregion
    }
    
    [DefaultClassOptions]
    [DomainComponent]
    public interface HierarchyNodeDescendant : Xafari.DC.HierarchyNode, Xafari.BC.DC.DynamicProperties.PersistentDynamicObject
    {
        [HierarchyNodeNameProperty()]
        String Name { get; set; }

        #region Default Hierarchy

        
        [ParentProperty()]
        HierarchyNodeDescendant Parent { get; set; }

        [BackReferenceProperty("Parent")]
        [ChildrenProperty()]
        IList<HierarchyNodeDescendant> Children { get; }

        #endregion

        #region Hierarchy1

        [ParentProperty("Hierarchy1")]
        HierarchyNodeDescendant Parent1 { get; set; }

        [BackReferenceProperty("Parent1")]
        [ChildrenProperty("Hierarchy1", DeletingMode = DeletingMode.Cascade)]
        IList<HierarchyNodeDescendant> Children1 { get; }

        [FieldSize(1000)]
        [ModelDefault("RowCount", "0")]
        [ModelDefault("AllowEdit", "false")]
        [HierarchyFullNameProperty("Hierarchy1")]
        string HierarchyFullName1 { get; set; }

        #endregion

        #region Hierarchy2
        [ParentProperty("Hierarchy2")]
        HierarchyNodeDescendant Parent2 { get; set; }

        [BackReferenceProperty("Parent2")]
        [ChildrenProperty("Hierarchy2", DeletingMode = DeletingMode.Replacement)]
        IList<HierarchyNodeDescendant> Children2 { get;}

        [FieldSize(1000)]
        [ModelDefault("RowCount", "0")]
        [ModelDefault("AllowEdit", "false")]
        [HierarchyFullNameProperty("Hierarchy2")]
        string HierarchyFullName2 { get; set; }

        #endregion

        #region Hierarchy3
        [ParentProperty("Hierarchy3")]
        HierarchyNodeDescendant Parent3 { get; set; }

        [BackReferenceProperty("Parent3")]
        [ChildrenProperty("Hierarchy3", DeletingMode = DeletingMode.Replacement)]
        IList<HierarchyNodeDescendant> Children3 { get; }

        [FieldSize(1000)]
        [ModelDefault("RowCount", "0")]
        [ModelDefault("AllowEdit", "false")]
        [HierarchyFullNameProperty("Hierarchy3")]
        string HierarchyFullName3 { get; set; }

        #endregion
    }

    [DefaultClassOptions]
    [DomainComponent]
    public interface HierarchicalClassifierItemDescendant : Xafari.BC.DC.HierarchicalClassifierItem
    {
        [ParentProperty]
        HierarchicalClassifierItemDescendant Parent { get; set; }

        [ChildrenProperty]
        [BackReferenceProperty("Parent")]
        IList<HierarchicalClassifierItemDescendant> Children { get; }

        [RuleFromBoolProperty("IsValidParentDC", DefaultContexts.Save, "Нельзя выбрать самого себя в вышестоящие")]
        [NonPersistentDc]
        [MemberDesignTimeVisibility(false)]
        [Browsable(false)]
        bool IsValidParent { get; }
    }

    [DomainLogic(typeof(HierarchicalClassifierItemDescendant))]
    public class DCHierarchyNodeObjectLogic : DomainLogicBase<HierarchicalClassifierItemDescendant>
    {
        public DCHierarchyNodeObjectLogic(HierarchicalClassifierItemDescendant instance) : base(instance) { }

        public static bool Get_IsValidParent(HierarchicalClassifierItemDescendant instance)
        {
            return HierarchyNodePersistentHelper.CheckCircularReference(instance);
        }
    }
}
